@@ -5,7 +5,7 @@
 #       University of Pavia - Italy                           #
 #       www.labmedinfo.org                                    #
 #                                                             #
-#   $Id: zzz.R 64 2007-12-17 14:32:33Z tonig $
+#   $Id: zzz.R 94 2008-01-08 16:44:46Z tonig $
 #                                                             #
 ###############################################################
 
@@ -16,5 +16,11 @@
 	    "\n",
             sep=""))
   library.dynam("dtw");
+  library(proxy);
+
+  ## Register DTW as a distance function into package proxy
+  pr_DB$set_entry(FUN=dtwpairdist, names="DTW", description="Dynamic Time Warping",
+                  loop=TRUE, formula="minimum of sum(x[xw[i]]-y[yw[i]]) over all xw, yw");
+
   invisible()
 }
